//// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//// PARTICULAR PURPOSE.
////
//// Copyright (c) Microsoft Corporation. All rights reserved

#pragma once

#include "pch.h"
#include "DirectXBase.h"


typedef enum {
    Bezier,
    Live
} RenderingMode;

typedef enum 
{
    None,
    Ink,    
    Erase
} PointerEventType;


ref class ShapeRenderer : public DirectXBase
{
internal:
    ShapeRenderer();

    virtual void CreateDeviceIndependentResources() override;
    virtual void CreateDeviceResources() override;
    virtual void CreateWindowSizeDependentResources() override;
    virtual void UpdateForWindowSizeChange() override;
    virtual void Render() override;
	virtual void Present() override;

    void OnPointerEntered(
            Platform::Object^ sender,
            Windows::UI::Xaml::Input::PointerRoutedEventArgs^ args);

	void OnPointerPressed(
            Platform::Object^ sender,
            Windows::UI::Xaml::Input::PointerRoutedEventArgs^ args);
            
    void OnPointerReleased(
        Platform::Object^ sender,
        Windows::UI::Xaml::Input::PointerRoutedEventArgs^ args);
            
    void OnPointerMoved(
        Platform::Object^ sender,
        Windows::UI::Xaml::Input::PointerRoutedEventArgs^ args);
    
	void OnDataRequested(
        _In_ Windows::ApplicationModel::DataTransfer::DataTransferManager^ sender,
        _In_ Windows::ApplicationModel::DataTransfer::DataRequestedEventArgs^ args
        );

    void BezierRender();

    // Public signature methods.
	void OnChangeDrawingAttributes(Windows::UI::Color newStrokeColor, Windows::Foundation::Size sz);

	property Platform::Agile<Windows::UI::Input::Inking::InkDrawingAttributes> DrawingAttributes
	{
		Platform::Agile<Windows::UI::Input::Inking::InkDrawingAttributes> get()
		{
			return m_drawingAttributes;
		}
	}

	property Windows::UI::Xaml::Controls::SwapChainBackgroundPanel^ SetSwapChain
	{
		void set(Windows::UI::Xaml::Controls::SwapChainBackgroundPanel^ value)
		{
			m_MainPageSwapChain = value;
		}
	}
    // Public saving methods.
    void SaveCanvas(Windows::Storage::Streams::IRandomAccessStream^ randomAccessStream, GUID wicFormat);

internal:
    void ConvertStrokeToGeometry(
        _In_ Windows::UI::Input::Inking::InkStroke^ stroke,
        _Outptr_ ID2D1PathGeometry** geometry);

private:

	Platform::String^ QueryPointer(Windows::UI::Input::PointerPoint^ ptrPt);
	void UpdateInfoPop(Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
	void DestroyInfoPop(Windows::UI::Xaml::Input::Pointer^ ptr);
	void CreateInfoPop(Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);

	void SaveBitmapToStream(
        _In_ ID2D1Bitmap1* d2dBitmap,
        _In_ IWICImagingFactory2* wicFactory2,
        _In_ ID2D1DeviceContext* d2dContext,
        _In_ REFGUID wicFormat,
        _In_ IStream* stream
        );

	void CreateBrush(
    _In_ ID2D1DeviceContext* d2dContext, 
    _In_ Windows::UI::Color color, 
    _Outptr_ ID2D1SolidColorBrush** brush);

	PointerEventType GetPointerEventType(_In_ Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
	bool EventIsErase(_In_ Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
	bool EventIsInk(_In_ Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);	

    Windows::UI::Xaml::Controls::SwapChainBackgroundPanel^  m_swapChainPanel;
	Platform::Collections::Map<UINT, Windows::UI::Xaml::Input::Pointer^>^ m_PointerContact;
	UINT														  m_activeContacts;
    
	// D2D data members
    Microsoft::WRL::ComPtr<ID3D11Texture2D>					m_currentBuffer;
    Microsoft::WRL::ComPtr<ID3D11Texture2D>					m_previousBuffer;
	Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>			m_backgroundBrush;
    Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>			m_inkBrush;
    Microsoft::WRL::ComPtr<ID2D1StrokeStyle>				m_inkStyle;
    

    Windows::UI::Color										m_backgroundColor;
	Windows::UI::Xaml::Controls::SwapChainBackgroundPanel^  m_MainPageSwapChain;
    // Inking members
    Platform::Agile<Windows::UI::Input::Inking::InkStrokeBuilder>			m_strokeBuilder;
    Platform::Agile<Windows::UI::Input::Inking::InkStrokeContainer>			m_strokeContainer;
    Platform::Agile<Windows::UI::Input::Inking::InkDrawingAttributes>		m_drawingAttributes;

	// Stores the id of the 'active' pointer, -1 if none. We are allowing only 
    // one 'active' pointer at a time. This variable is set by OnPointerPressed,
    // checked by OnPointerMoved, and reset by OnPointerReleased.
    int														m_pointerId;

    Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^	m_manipulationPoints;
    Windows::UI::Input::Inking::InkManipulationMode							m_manipulationMode;

    RenderingMode															m_renderingMode;

};
